package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestLDI {

  private static final int BYTE_VALUE = 0x40;
  private final String MNEMONIC = "LDI";

  @Test
  void testMnemonic() throws SyntaxError {
    LDI mnemonic = new LDI(MNEMONIC + " 0x00");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());

    mnemonic = new LDI(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x01, mnemonic.getByte());

    mnemonic = new LDI(MNEMONIC + " 0x0f");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDI mnemonic1 = new LDI(MNEMONIC);
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      LDI mnemonic1 = new LDI(MNEMONIC + " 0x10");
      mnemonic1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    assertEquals(LDI.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

  }
}
